<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Observer;

use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\Product;
use Flat101\EnhancedEcommerce\Model\Session;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Quote\Model\Quote\Item;
use Magento\Framework\Serialize\SerializerInterface;
use Psr\Log\LoggerInterface;
use Magento\Checkout\Model\Session as CheckoutSession;

class AddToCartObserver implements ObserverInterface
{
    /** @var Product */
    private $productGenerator;

    /** @var Session */
    private $sessionManager;

    /** @var SerializerInterface */
    protected $serializer;

    /** @var LoggerInterface */
    private $logger;

    /** @var CheckoutSession */
    private $checkoutSession;

    public function __construct(
        Product $productGenerator,
        Session $sessionManager,
        SerializerInterface $serializer,
        LoggerInterface $logger,
        CheckoutSession $checkoutSession
    ) {
        $this->productGenerator = $productGenerator;
        $this->sessionManager = $sessionManager;
        $this->serializer = $serializer;
        $this->logger = $logger;
        $this->checkoutSession = $checkoutSession;
    }

    /**
     * {@inheritdoc}
     */
    public function execute(Observer $observer): void
    {
        try {
            /** @var Item $item */
            $product = $observer->getEvent()->getData('product');
            $qty = $observer->getEvent()->getData('request')->getParam('qty') ?: 1;
            $productExtraInfo = ($observer->getEvent()->getData('request')->getParam('click_data')) ? $this->serializer->unserialize($observer->getEvent()->getData('request')->getParam('click_data')) : null;

            $item = $this->getItemFromQuote($product->getId());

            $product = $this->productGenerator->generate($product, $qty, $item, $productExtraInfo);

            $this->sessionManager->setAddProductToCart($product);
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        }
    }

    /**
     * @param $productId
     * @return Item|null
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getItemFromQuote($productId): ?Item
    {
        $quote = $this->checkoutSession->getQuote();

        // TODO: When user adds same option with other between, wrong result
        $allItems = array_reverse($quote->getAllItems());
        foreach ($allItems as $item) {
            $itemInfo = $item->getProduct()->getTypeInstance(true)->getOrderOptions($item->getProduct());

            if ((array_key_exists('info_buyRequest', $itemInfo) && is_array($itemInfo['info_buyRequest'])) &&
                (array_key_exists('attributes_info', $itemInfo) && is_array($itemInfo['attributes_info'])) &&
                ($productId == $itemInfo['info_buyRequest']['product'])) {
                return $item;
            }
        }

        return null;
    }
}
